"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CliTable = void 0;
const cli_table_1 = __importDefault(require("cli-table"));
const colors_1 = __importDefault(require("colors"));
const logger_1 = require("./logger");
class CliTable {
    constructor(title, config, style = CliTable.DEFAULT_STYLE) {
        this.title = title;
        this.config = config;
        this.style = style;
        this.table = this.createTable(this.getHeadings());
    }
    getHeadings() {
        return Object.values(this.config.headings).map((item) => item.display);
    }
    createTable(headings) {
        return new cli_table_1.default({
            head: headings,
            chars: this.style,
        });
    }
    wrapText(text, width) {
        const regex = new RegExp(`.{1,${width}}`, 'g');
        const matches = text.match(regex);
        return matches ? matches.join('\n') : text;
    }
    print(data) {
        data.forEach((record) => {
            const row = this.buildRow(record);
            this.addRowToTable(row);
        });
        this.displayTable();
    }
    buildRow(record) {
        const keys = Object.keys(this.config.headings);
        return keys.map((key) => this.applyStyle(this.transform(record, key), key));
    }
    transform(record, key) {
        const type = this.config.headings[key].type;
        const width = this.config.headings[key].width || 0;
        const truncate = this.config.headings[key].truncate || false;
        const truncateLength = this.config.headings[key].truncateLength || 50;
        if (type === 'datetime') {
            const date = new Date(record[key]);
            if (!isNaN(date.getTime())) {
                return date.toISOString().replace('T', ' ').substring(0, 19);
            }
        }
        else if (typeof record[key] === 'string') {
            let formattedText = record[key] || 'N/A';
            if (truncate && formattedText.length > truncateLength) {
                formattedText = formattedText.substring(0, truncateLength - 3) + '...';
            }
            if (width > 0) {
                formattedText = this.wrapText(formattedText, width);
            }
            return formattedText;
        }
        return String(record[key] || 'N/A');
    }
    applyStyle(text, key) {
        const style = this.config.headings[key].style;
        if (key === 'id') {
            return colors_1.default.blue.bold(text);
        }
        if (style) {
            if (style.color) {
                return style.color(text);
            }
        }
        return text;
    }
    addRowToTable(row) {
        this.table.push(row);
    }
    displayTable() {
        (0, logger_1.success)(`\n\n${this.title}`);
        (0, logger_1.log)(this.table.toString());
    }
}
exports.CliTable = CliTable;
CliTable.DEFAULT_STYLE = {
    top: '═',
    'top-mid': '╤',
    'top-left': '╔',
    'top-right': '╗',
    bottom: '═',
    'bottom-mid': '╧',
    'bottom-left': '╚',
    'bottom-right': '╝',
    left: '║',
    'left-mid': '╟',
    mid: '─',
    'mid-mid': '┼',
    right: '║',
    'right-mid': '╢',
    middle: '│',
};
